"""
Launch server with: bokeh serve --show --allow-websocket-origin '*' launch_server.py

There are 3 sections of code below.

section 1: No print statements are observed in Flask or Bokeh windows. JS print statements are
seen in the browser.

section 2: Print statement shows in bokeh terminal, nothing in flask terminal.

section 3: Same as 2.



"""
from flask import Flask, render_template
from bokeh.client import pull_session
from bokeh.models import CustomJS
from bokeh.embed import server_session, server_document
import bokeh_layout


# Initiate Flask App
app = Flask(__name__)
app.secret_key = """
                \xd4W\x18
                """


def button_callback_2():
    print('Test button 2 triggered.')


@app.route('/')
def create_session():
    """
    _________________________________________________________________________________
    This section will replicate the error I am seeing - i.e. .on_click() event appears
    to be blocked. No print statements appear in flask or bokeh window.

    Comment this section out to test other sections.
    """
    # Create new, personalized layout
    new_layout, test_button_2 = bokeh_layout.create_layout()
    test_button_2.on_click(button_callback_2)
    test_button_2.js_on_click(CustomJS(code="""console.log('Test button 2 clicked!')"""))

    # Pull & personalize a session for a user when they log in
    bokeh_session = pull_session(url="http://localhost:5006/launch_server")
    bokeh_session.document.clear()
    bokeh_session.document.add_root(new_layout)
    session_id = bokeh_session.id
    bokeh_session.close()

    # Later in workflow, pull this users personalized session and create script
    script = server_session(None, session_id, url="http://localhost:5006/launch_server")
    return render_template("test.html", script=script)

    # """
    # ____________________________________________________________________________
    # This section forgoes the customization of the session via clearing document and adding a
    # new root. The print output doesn't show up in Flask app but it does show up in Bokeh Server.
    #
    # Comment this section out to test other sections.
    # """
    # # Pull & personalize a session for a user when they log in
    # bokeh_session = pull_session(url="http://localhost:5006/launch_server")
    # session_id = bokeh_session.id
    # bokeh_session.close()
    #
    # # Later in workflow, pull this users personalized session and create script
    # script = server_session(None, session_id, url="http://localhost:5006/launch_server")
    # return render_template("test.html", script=script)

    # """
    # _________________________________________________________________________
    # Similarly, this causes the print statements to appear in the Bokeh Server.
    #
    # Comment this section out to test other sections.
    # """
    # Create new layout
    # new_layout, test_button_2 = bokeh_layout.create_layout()
    # test_button_2.on_click(button_callback_2)
    #
    # Run directly from server document without calling a session.
    # script = server_document(url="http://localhost:5006/launch_server")
    # return render_template("test.html", script=script)


if __name__ == '__main__':
    app.run(port=8000)
